@@ -40,10 +40,6 @@
 #include <fcntl.h>
 #ifdef _WIN32
 #include <winsock2.h>
-#ifdef HAVE_BLUETOOTH
-#include <ws2bth.h>
-#define bdaddr_t	BTH_ADDR
-#endif
 #define S_IRGRP 0
 #define S_IROTH 0
 #define S_IWGRP 0
@@ -77,9 +73,6 @@
 #define CUR_DIR				"./"
 
 
-#ifdef HAVE_BLUETOOTH
-static bdaddr_t *bt_src = NULL;
-#endif
 static char *device = NULL;
 static int channel = 10; /* OBEX_PUSH_HANDLE */
 
@@ -220,7 +213,7 @@ inline static void FL_XML_BODY_FILENAME(struct rawdata_stream *stream, char *nam
 
 inline static void FL_XML_BODY_SIZE(struct rawdata_stream *stream, unsigned int size)	
 {
-	const char* format = "size=\"%d\" ";
+	const char format[] = "size=\"%d\" ";
 	char str_size[sizeof(format)+14];
 
 	snprintf(str_size,sizeof(str_size), format, size);
@@ -229,7 +222,7 @@ inline static void FL_XML_BODY_SIZE(struct rawdata_stream *stream, unsigned int
 
 inline static void FL_XML_BODY_PERM(struct rawdata_stream *stream, mode_t file, mode_t dir)	
 {
-	const char* format = "user-perm=\"%s%s%s\" ";
+	const char format[] = "user-perm=\"%s%s%s\" ";
 	char str_perm[sizeof(format)];
 
 	snprintf(str_perm,sizeof(str_perm),format,
@@ -888,7 +881,7 @@ reset:
 	       	break;
 #ifdef HAVE_BLUETOOTH
        	case OBEX_TRANS_BLUETOOTH:
-		if (0 > BtOBEX_ServerRegister(handle, bt_src, channel)) {
+		if (0 > BtOBEX_ServerRegister(handle, /*bdaddr_t *bt_src*/NULL, channel)) {
        			perror("failed to register bluetooth server");
 	       		exit(-1);
 		}
@@ -930,7 +923,10 @@ reset:
 	if (use_sdp)
 	{
 		fprintf(stderr, "sdp unregister\n");
-		obexftp_sdp_unregister_ftp();
+		if (0 > obexftp_sdp_unregister_ftp())
+		{
+       			fprintf(stderr, "unregister from SDP Server failed.\n");
+		}
 	}
 
 }
