@@ -1,43 +1,39 @@
-/*
- *  obexftp/cache.c: ObexFTP client library
- *
- *  Copyright (c) 2002 Christian W. Zuckschwerdt <zany@triq.net>
- *
- *  This program is free software; you can redistribute it and/or modify it
- *  under the terms of the GNU General Public License as published by the Free
- *  Software Foundation; either version 2 of the License, or (at your option)
- *  any later version.
- *
- *  This program is distributed in the hope that it will be useful, but
- *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- *  for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *     
+/**
+	\file obexftp/cache.c
+	ObexFTP client API caching layer.
+	ObexFTP library - language bindings for OBEX file transfer.
+
+	Copyright (c) 2002-2007 Christian W. Zuckschwerdt <zany@triq.net>
+
+	ObexFTP is free software; you can redistribute it and/or modify
+	it under the terms of the GNU Lesser General Public License as
+	published by the Free Software Foundation; either version 2 of
+	the License, or (at your option) any later version.
+
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU Lesser General Public License for more details.
+
+	You should have received a copy of the GNU Lesser General Public
+	License along with ObexFTP. If not, see <http://www.gnu.org/>.
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-#include <string.h>
+#define _GNU_SOURCE
+#include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <string.h>
 #include <sys/stat.h> /* __S_IFDIR, __S_IFREG */
 #ifndef S_IFDIR
 #define S_IFDIR	__S_IFDIR
 #endif
 #ifndef S_IFREG
-#define S_IFDIR	__S_IFREG
-#endif
-
-#ifdef HAVE_ICONV
-#include <iconv.h>
-#include <locale.h>
-#include <langinfo.h>
+#define S_IFREG	__S_IFREG
 #endif
 
 #include <openobex/obex.h>
@@ -45,21 +41,22 @@
 #include "obexftp.h"
 #include "client.h"
 #include "object.h"
+#include "unicode.h"
 #include "cache.h"
 
 #include <common.h>
 
 
-/*
- * Normalize the path argument, add/remove leading/trailing slash
- * turns relative paths into (most likely wrong) absolute ones
- * wont expand "../" or "./"
+/**
+	Normalize the path argument, add/remove leading/trailing slash
+	turns relative paths into (most likely wrong) absolute ones
+	wont expand "../" or "./".
  */
 static /*@only@*/ char *normalize_dir_path(int quirks, const char *name)
 {
 	char *copy, *p;
 
-	return_val_if_fail(name != NULL, strdup(""));
+	if (!name) name = "";
 
 	p = copy = malloc(strlen(name) + 2); /* at most add two slashes */
 
@@ -87,13 +84,14 @@ static /*@only@*/ char *normalize_dir_path(int quirks, const char *name)
 }
 
 
-/*
- * methods that need to invalidate cache lines:
- * setpath (when create is on)
- * put
- * put_file
- * del
- * rename
+/**
+	Purge all cache object at/below a given path.
+	Methods that need to invalidate cache lines:
+	- setpath (when create is on)
+	- put
+	- put_file
+	- del
+	- rename
  */
 void cache_purge(cache_object_t **root, const char *path)
 {
@@ -106,6 +104,8 @@ void cache_purge(cache_object_t **root, const char *path)
 				free(node->name); \
 			if (node->content) \
 				free(node->content); \
+			if (node->stats) \
+				free(node->stats); \
 			free(node); \
 			} while(0)
 
@@ -140,6 +140,9 @@ void cache_purge(cache_object_t **root, const char *path)
 	free(pathonly);
 }
 
+/**
+	Retrieve an object from the cache.
+ */
 int get_cache_object(const obexftp_client_t *cli, const char *name, char **object, int *size)
 {
 	cache_object_t *cache;
@@ -159,6 +162,9 @@ int get_cache_object(const obexftp_client_t *cli, const char *name, char **objec
 	return -1;
 }
 
+/**
+	Store an object in the cache.
+ */
 int put_cache_object(obexftp_client_t *cli, /*@only@*/ char *name, /*@only@*/ char *object, int size)
 {
 	cache_object_t *cache;
@@ -176,7 +182,9 @@ int put_cache_object(obexftp_client_t *cli, /*@only@*/ char *name, /*@only@*/ ch
 	return 0;
 }
 
-/* List a directory from cache, optionally loading it first. */
+/**
+	List a directory from cache, optionally loading it first.
+ */
 static char *obexftp_cache_list(obexftp_client_t *cli, const char *name)
 {
 	char *path, *listing;
@@ -213,12 +221,15 @@ static char *obexftp_cache_list(obexftp_client_t *cli, const char *name)
 
 /* simple xml parser */
 
+/**
+	Parse fixed format date string to time_t.
+ */
 static time_t atotime (const char *date)
 {
 	struct tm tm;
 
 	if (6 == sscanf(date, "%4d%2d%2dT%2d%2d%2d",
-			&tm.tm_year, &tm.tm_mday, &tm.tm_mon,
+			&tm.tm_year, &tm.tm_mon, &tm.tm_mday,
 			&tm.tm_hour, &tm.tm_min, &tm.tm_sec)) {
 		tm.tm_year -= 1900;
 		tm.tm_mon--;
@@ -228,8 +239,12 @@ static time_t atotime (const char *date)
 	return mktime(&tm);
 }
 
-/* very limited - not multi-byte character save */
-/* it is actually "const char *xml" but can't be declared as such */
+/**
+	Parse an XML file to array of stat_entry_t's.
+	Very limited - not multi-byte character save.
+	It's actually "const char *xml" but can't be declared as such.
+	\return a new allocated array of stat_entry_t's.
+ */
 static stat_entry_t *parse_directory(char *xml)
 {
         const char *line;
@@ -240,34 +255,19 @@ static stat_entry_t *parse_directory(char *xml)
         char size[201]; // int would be ok too.
 
 	stat_entry_t *dir_start, *dir;
-	int i;
-#ifdef HAVE_ICONV
-	iconv_t utf8;
-	char *xml_latin1, *xml_end;
-	size_t ni, no, nrc;
-	int ret;
-#endif
+	int ret, n, i;
+	char *xml_conv;
 		
 	if (!xml)
 		return NULL;
-#ifdef HAVE_ICONV
-	setlocale(LC_CTYPE, "");
-	DEBUG(2, "Iconv to %s\n", nl_langinfo(CODESET));
-	utf8 = iconv_open (nl_langinfo(CODESET), "UTF-8");
-	ni = strlen(xml);
-	no = ni;
-	xml_latin1 = xml_end = malloc(no);
-	p = xml;
-	if (xml_latin1) {
-		nrc = iconv (utf8, (char* *)&p, &ni, &xml_end, &no);
-		ret = iconv_close (utf8);
-		if (nrc != (size_t)(-1)) {
-			xml = xml_latin1;
-		} else {
-			DEBUG(1, "Iconv conversion error: '%s'\n", p);
-		}
-	}
-#endif
+	n = strlen(xml) + 1;
+	xml_conv = malloc(n);
+	ret = Utf8ToChar(xml_conv, xml, n);
+       	if (ret > 0) {
+       		xml = xml_conv;
+       	} else {
+       		DEBUG(1, "UTF-8 conversion error\n");
+       	}
 	
 	DEBUG(4, "Converted cache xml: '%s'\n", xml);
 	/* prepare a cache to hold this dir */
@@ -322,22 +322,23 @@ static stat_entry_t *parse_directory(char *xml)
 
 	dir->name[0] = '\0';
 
-#ifdef HAVE_ICONV
-	if (xml_latin1)
-		free (xml_latin1);
-#endif
+	if (xml_conv)
+		free (xml_conv);
+
         return dir_start;
 }
 
 
 /* directory handling */
 
-typedef struct dir_stream dir_stream_t;
-struct dir_stream {
+typedef struct {
 	stat_entry_t *cur;
 	/* stat_entry_t *head; -- so we can free this? */
-};
+} dir_stream_t;
 
+/**
+	Prepare a directory for reading.
+ */
 void *obexftp_opendir(obexftp_client_t *cli, const char *name)
 {
 	cache_object_t *cache;
@@ -365,7 +366,10 @@ void *obexftp_opendir(obexftp_client_t *cli, const char *name)
 	return (void *)stream;
 }
 
-/* the stat entry is a cache object so we do nothing */
+/**
+	Close a directory after reading.
+	The stat entry is a cache object so we do nothing.
+ */
 int obexftp_closedir(void *dir) {
 	if (!dir)
 		return -1;
@@ -373,6 +377,9 @@ int obexftp_closedir(void *dir) {
 	return 0;
 }
 
+/**
+	Read the next entry from an open directory.
+ */
 stat_entry_t *obexftp_readdir(void *dir) {
 	dir_stream_t *stream;
 	
@@ -386,6 +393,9 @@ stat_entry_t *obexftp_readdir(void *dir) {
 	return stream->cur++;
 }
 	 
+/**
+	Stat a directory entry.
+ */
 stat_entry_t *obexftp_stat(obexftp_client_t *cli, const char *name)
 {
 	cache_object_t *cache;
@@ -400,8 +410,10 @@ stat_entry_t *obexftp_stat(obexftp_client_t *cli, const char *name)
 	if (p) {
 		*p++ = '\0';
 		basename = p;
-	} else
+	} else {
+		*path = '\0';
 		basename = name;
+	}
 	DEBUG(2, "%s() stating '%s' / '%s'\n", __func__, path, basename);
 
 	/* fetch dir if needed */
@@ -411,8 +423,10 @@ stat_entry_t *obexftp_stat(obexftp_client_t *cli, const char *name)
 	abs = normalize_dir_path(cli->quirks, path);
 	for (cache = cli->cache; cache && strcmp(cache->name, abs); cache = cache->next);
 	free(abs);
-	if (!cache)
+	if (!cache) {
+		free(path);
 		return NULL;
+	}
 	DEBUG(2, "%s() found '%s'\n", __func__, cache->name);
 		 
 	/* read dir */
@@ -422,6 +436,7 @@ stat_entry_t *obexftp_stat(obexftp_client_t *cli, const char *name)
 	
 	/* then lookup the basename */
 	for (entry = cache->stats; entry && *entry->name && strcmp(entry->name, basename); entry++);
+	free(path);
 	if (!entry || !(*entry->name))
 		return NULL;
 
