@@ -51,6 +51,8 @@
 
 #ifdef HAVE_BLUETOOTH
 #include "bt_kit.h"
+#else
+#define ESOCKTNOSUPPORT	WSAESOCKTNOSUPPORT
 #endif /* HAVE_BLUETOOTH */
 
 #include <openobex/obex.h>
@@ -392,6 +394,7 @@ static void cli_obex_event(obex_t *handle, obex_object_t *object, int UNUSED(mod
 	case OBEX_EV_LINKERR:
 		cli->finished = TRUE;
 		cli->success = FALSE;
+		DEBUG(2, "%s() OBEX_EV_LINKERR\n", __func__);
 		break;
 	
 	case OBEX_EV_STREAMEMPTY:
@@ -422,8 +425,10 @@ static int obexftp_sync(obexftp_client_t *cli)
 		ret = OBEX_HandleInput(cli->obexhandle, 20);
 		DEBUG(3, "%s() OBEX_HandleInput = %d\n", __func__, ret);
 
-		if (ret <= 0)
+		if (ret <= 0) {
+			DEBUG(2, "%s() OBEX_HandleInput error: %d\n", __func__, errno);
 			return -1;
+		}
 	}
 
 	DEBUG(3, "%s() Done success=%d\n", __func__, cli->success);
@@ -431,7 +436,7 @@ static int obexftp_sync(obexftp_client_t *cli)
 	if(cli->success)
 		return 1;
 	else
-		return -1;
+		return - cli->obex_rsp;
 }
 
 
@@ -559,6 +564,24 @@ int obexftp_connect_uuid(obexftp_client_t *cli, const char *device, int port, co
 }
 
 
+int obexftp_connect_service(obexftp_client_t *cli, const char *src, const char *device, int port, int service)
+{
+	uint8_t *uuid = NULL;
+	uint32_t uuid_len = 0;
+	if (service == OBEX_FTP_SERVICE) {
+		uuid = UUID_FBS;
+		uuid_len = sizeof(UUID_FBS);
+	}
+	if (service == OBEX_SYNC_SERVICE) {
+		uuid = UUID_IRMC;
+		uuid_len = sizeof(UUID_IRMC);
+	}
+	// otherwiese default to OBEX_PUSH_SERVICE
+	return obexftp_connect_src(cli, src, device, port, uuid, uuid_len);
+}
+
+
+
 /**
 	Connect this ObexFTP client using a given source address by sending an OBEX CONNECT request.
 
@@ -642,6 +665,9 @@ int obexftp_connect_src(obexftp_client_t *cli, const char *src, const char *devi
 		else if (!strncmp(src, "hci", 3)) {
 			hci_devba(atoi(src + 3), &src_addr);
 		}
+		else if (atoi(src) != 0) {
+			hci_devba(atoi(src), &src_addr);
+		}
 #endif
 		else {
 			str2ba(src, &src_addr);
